--[[
	WhiteGuardian v0.2
	Author: Dylan Coakley
]]--

local sti = require "libs.STI"
require("libs/AnAL")
require "classes/Player"
require "classes/Item"
require "classes/Menu"

smallfont = love.graphics.newFont( 14 )
largefont = love.graphics.newFont( 18 )

-- Simple box collision check between two rectangles A and B
function box_collision(xa, ya, wa, ha, xb, yb, wb, hb)
	return ((xa <= xb + wb) and (xb <= xa + wa) and (ya <= yb + hb) and (yb <= ya + ha))
end

function maketransparent(file)
   local imgdata = love.image.newImageData(file)
   for i=0,imgdata:getWidth()-1 do
      for j=0,imgdata:getHeight()-1 do
         local r,g,b,a = imgdata:getPixel(i,j)
         if r == 43 and g == 133 and b == 133 and a == 255 then
            imgdata:setPixel(i,j,255,255,255,0)
         end
      end
   end
   return love.graphics.newImage(imgdata)
end

function addToInv(itemToAdd)
    local addToSlot = #inventory + 1
    for i = 1, #inventory do
        if inventory[i].name == itemToAdd.name then
            addToSlot = i
            inventory[addToSlot].amount = inventory[addToSlot].amount + itemToAdd.amount
            return
        end
    end
    table.insert(inventory, itemToAdd)
end

function getCoordsByPercent(percentW, percentH)
	width, height = love.graphics.getDimensions( )

	w = (width/100)*percentW
	--print(w)
	h = (height/100)*percentH
	--print(h)

	return w,h
end

function getCoordsByPercentX(percentW)
	width, height = love.graphics.getDimensions( )

	w = (width/100)*percentW

	return w
end

function getCoordsByPercentY(percentH)
	width, height = love.graphics.getDimensions( )

	h = (height/100)*percentH

	return h
end

function love.load()
	--print(sti.version)
	

	selectedP = "jake"
	selectedPClass = "Knight"

	-- Create the player
	plrX, plrY = getCoordsByPercent(50,50)
	player = Player:new(plrX-16, plrY-24, 32, 50, "Jake",selectedP,selectedPClass)

	inventory = {

	}

	party = {

	}
	table.insert(party, player)
	itemw, itemh = getCoordsByPercent(50,50)
	-- Create the items on the map
	-- x,y,w,h,itemID,itemName,fileName,atk,def,specialPercent,specialType, itemAmount
	itemsOnMap = {
		Item:new(itemw,itemh,32, 32, 1, "Wooden Bat","woodenBat",2,nil, 0, "", 1),
		Item:new(300,50,32, 32, 2, "Ox Ring","oxRing",nil,nil, 2, "Strength", 1),
		Item:new(70,250,32, 32, 3, "Rusty Dagger", "rustyIronDagger",2,nil, 0, "", 1),
		Item:new(60,110,32, 32, 1, "bronzeSpear","bronzeSpear",2,nil, 0, "", 1),
		Item:new(390,70,32, 32, 2, "Cloth Robe","clothRobe",nil,1, 0, "", 1),
		Item:new(100,250,32, 32, 3, "Pine Plank Shield", "pinePlankShield",nil,2, 0, "", 1),
		Item:new(80,100,32, 32, 1, "Wooden Bat","woodenBat",2,nil, 0, "", 1),
		Item:new(350,50,32, 32, 2, "Ox Ring","oxRing",2,nil, 2, "Strength", 1),
		Item:new(90,250,32, 32, 3, "Rusty Dagger", "rustyIronDagger",2,nil, 0, "", 1),
		Item:new(20,110,32, 32, 1, "bronzeSpear","bronzeSpear",2,nil, 0, "", 1),
		Item:new(10,70,32, 32, 2, "Cloth Robe","clothRobe",nil,1, 0, "", 1),
		Item:new(50,250,32, 32, 3, "Pine Plank Shield", "pinePlankShield",nil,2, 0, "", 1)
	}

	i		= 1
	speed	= 256

	-- Load multiple maps
	maps = {
		sti.new("assets/maps/rpgmap")
	}

	col = {}

	for k=1, #maps do
		local layer = 2
		

		-- Add a Custom Layer
		maps[k]:addCustomLayer("Sprite Layer", layer)

		local spriteLayer = maps[k].layers["Sprite Layer"]
		local image = maketransparent(player.image)

		standing = newAnimation(image, 128, 192, 0.1, -1)
		standing:addFrame(0, 0, 32, 48, 0.1)

		walkLeft = newAnimation(image, 128, 192, 0.1, -1)
		walkLeft:addFrame(0, 48, 32, 48, 0.1)
		walkLeft:addFrame(32, 48, 32, 48, 0.1)
		walkLeft:addFrame(64, 48, 32, 48, 0.1)
		walkLeft:addFrame(96, 48, 32, 48, 0.1)

		walkRight = newAnimation(image, 128, 192, 0.1, -1)
		walkRight:addFrame(0, 96, 32, 48, 0.1)
		walkRight:addFrame(32, 96, 32, 48, 0.1)
		walkRight:addFrame(64, 96, 32, 48, 0.1)
		walkRight:addFrame(96, 96, 32, 48, 0.1)

		walkDown = newAnimation(image, 128, 192, 0.1, -1)
		walkDown:addFrame(0, 0, 32, 48, 0.1)
		walkDown:addFrame(32, 0, 32, 48, 0.1)
		walkDown:addFrame(64, 0, 32, 48, 0.1)
		walkDown:addFrame(96, 0, 32, 48, 0.1)

		walkUp = newAnimation(image, 128, 192, 0.1, -1)
		walkUp:addFrame(0, 144, 32, 48, 0.1)
		walkUp:addFrame(32, 144, 32, 48, 0.1)
		walkUp:addFrame(64, 144, 32, 48, 0.1)
		walkUp:addFrame(96, 144, 32, 48, 0.1)

		-- Add Custom Data
		spriteLayer.sprite = {
			--image = love.graphics.newImage("assets/sprites/"..sprite..".png"),  this draws the whole spritesheet, not what I need
			x = 495,
			y = 462,
			r = 0,
		}
	end
	Menu:load()
end

function love.update(dt)

	-- Check for collisions between the player and the items on the map
	for i = 1, #itemsOnMap do
		local item = itemsOnMap[i]
		if item.health > 0 then
			-- If the player collide with an item on the map decrease health of item
			if box_collision(player.x, player.y, player.w, player.h, item.x, item.y, item.w, item.h) then
				itemsOnMap[i].health = itemsOnMap[i].health - 1
				addToInv(itemsOnMap[i])
			end
		end
	end


	s_w,s_h =  love.window.getDimensions( )
	-- Cap speed
	if speed < 128	then speed = 128 end
	if speed > 512	then speed = 512 end

	-- Update sprite's coordinates
	local sprite = maps[i].layers["Sprite Layer"].sprite
	down = love.keyboard.isDown

	if showMenu == false then
		if down("w") or down("up") then

			-- Update positions for items on map
			for i = 1, #itemsOnMap do
				if itemsOnMap[i].health > 0 then
					itemsOnMap[i].y =  itemsOnMap[i].y + speed * dt
				end
			end

			sprite.y = sprite.y - speed * dt
			walkDown:stop()
			walkUp:play()
			walkUp:update(dt)
		end
		if down("s") or down("down") then

			-- Update positions for items on map
			for i = 1, #itemsOnMap do
				if itemsOnMap[i].health > 0 then
					itemsOnMap[i].y =  itemsOnMap[i].y - speed * dt
				end
			end

			sprite.y = sprite.y + speed * dt
			walkUp:stop()
			walkDown:play()
			walkDown:update(dt)
		end
		if down("a") or down("left") then

			-- Update positions for items on map
			for i = 1, #itemsOnMap do
				if itemsOnMap[i].health > 0 then
					itemsOnMap[i].x =  itemsOnMap[i].x + speed * dt
				end
			end

			sprite.x = sprite.x - speed * dt
			walkRight:stop()
			walkLeft:play()
			walkLeft:update(dt)
		end
		if down("d") or down("right") then

			-- Update positions for items on map
			for i = 1, #itemsOnMap do
				if itemsOnMap[i].health > 0 then
					itemsOnMap[i].x =  itemsOnMap[i].x - speed * dt
				end
			end

			sprite.x = sprite.x + speed * dt
			walkLeft:stop()
			walkRight:play()
			walkRight:update(dt)
		 end

		for _, map in pairs(maps) do
			map:update(dt)
		end
	end
end

function love.draw()

	local sprite = maps[i].layers["Sprite Layer"].sprite
	local ww = love.graphics.getWidth()
	local wh = love.graphics.getHeight()
	local tx = math.floor(-sprite.x + ww / 2 - 16)
	local ty = math.floor(-sprite.y + wh / 2 - 16)

	-- Draw sprite in centre of screen
	love.graphics.push()
	love.graphics.translate(tx, ty)
	maps[i]:setDrawRange(tx, ty, ww, wh)
	maps[i]:draw()
	--maps[i]:drawCollisionMap(col[i])
	love.graphics.pop()
	x = (s_w/2) - 16
	y = (s_h/2) - 24

	player.x, player.y = x,y

	-- Draw the items on the map if not collected
	for i = 1, #itemsOnMap do
		local item = itemsOnMap[i]
		if item.health > 0 then
			love.graphics.draw(item.image, item.x, item.y)
		end
	end
	-- Player movement animations
	if down("a") or down("left") then
		walkLeft:draw(x, y)
	end

	if down("d") or down("right") then
		walkRight:draw(x, y)
	end

	if down("s") or down("down") then
		walkDown:draw(x, y)
	end

	if down("w") or down("up") then
		walkUp:draw(x, y)
	end

	if ((down("a") or down("left"))==false) and ((down("d") or down("right"))==false) and ((down("s") or down("down"))==false) and ((down("w") or down("up"))==false) then
		standing:draw(x, y)
	end

	Menu:draw()

	-- Draw the bounding rectangle for the player
	love.graphics.setColor(255,255,255)
	love.graphics.rectangle("line",player.x, player.y, player.w, player.h)

	-- Draw the bounding rectangles for the items on the map
	for i = 1, #itemsOnMap do
		local item = itemsOnMap[i]
		if item.health > 0 then
			love.graphics.rectangle("line",item.x, item.y, item.w, item.h)
		end
	end
end

function love.keypressed(key, isrepeat)
	if key == "escape" then
		love.event.quit()
	end
	Menu:keypressed(key, isrepeat)
end

function love.resize(w, h)
	for k=1, #maps do
		maps[k]:resize(w, h)
	end
	showMenu = false
end
